#include <GMLuaModule.h>
#include <filesystem.h>

GMOD_MODULE(Load, Unload);

ILuaInterface *gLua = NULL;
IFileSystem *filesystem = NULL;

void FileSystemLogging(const char *fileName, const char *accessType)
{
	ILuaObject *hookT = gLua->GetGlobal("hook");
		gLua->Push(hookT->GetMember("Call"));

		gLua->Push("FileSystemUsage");
		gLua->PushNil();
		gLua->Push(fileName);
		gLua->Push(accessType);

		gLua->Call(4, 0);
	hookT->UnReference();
}

int Load(lua_State *L)
{
	gLua = Lua();

	CreateInterfaceFn fileSystemFactory = Sys_GetFactory("filesystem_steam.dll");
	
	if (!fileSystemFactory)
		gLua->Error("Failed to retrieve required factory");
	
	filesystem = (IFileSystem *)fileSystemFactory(FILESYSTEM_INTERFACE_VERSION, NULL);

	if (filesystem)
		filesystem->AddLoggingFunc(FileSystemLogging);
	else
		gLua->Error("Failed to retrieve IFileSystem");

	return 0;
}

int Unload(lua_State *L)
{
	if (filesystem)
		filesystem->RemoveLoggingFunc(FileSystemLogging);

	return 0;
}